package partitioner

import (
	"fmt"
	"forever-files/db"
	"forever-files/types"
	"github.com/dustin/go-humanize"
)

func CalculatePartitions(store db.DB, targetSize int64) (partitions [][]types.FileMetadata, err error) {
	totalSize, err := store.GetTotalSize()
	if err != nil {
		return nil, fmt.Errorf("error getting total size: %w", err)
	}
	if targetSize <= 0 {
		targetSize = totalSize / 2
	}
	fmt.Printf("Total Size: %v\n", totalSize)
	fmt.Printf("Target Size: %v\n", targetSize)

	files, err := store.GetFiles()
	if err != nil {
		return nil, fmt.Errorf("error getting files: %w", err)
	}
	partitions = make([][]types.FileMetadata, 0)
	partitionSize := int64(0)
	partitionFiles := make([]types.FileMetadata, 0)
	overSizedFiles := make([]types.FileMetadata, 0)
	overSizedSize := int64(0)
	for _, file := range files {
		if partitionSize+file.Size > targetSize {
			//fmt.Printf("Partition Size: %v\n", humanize.Bytes(uint64(partitionSize)))
			partitions = append(partitions, partitionFiles)
			partitionFiles = make([]types.FileMetadata, 0)
			partitionSize = 0
		}
		if partitionSize < targetSize && partitionSize+file.Size < targetSize {
			partitionFiles = append(partitionFiles, file)
			partitionSize += file.Size
		} else {
			overSizedFiles = append(overSizedFiles, file)
			overSizedSize += file.Size
		}
	}
	if len(partitionFiles) > 0 {
		partitions = append(partitions, partitionFiles)
	}

	//for _, partition := range partitions {
	//	fmt.Printf("Partition File Count: %v\n", len(partition))
	//}
	fmt.Printf("Over Sized File Count: %v\n", len(overSizedFiles))
	fmt.Printf("Total Over Sized Size: %v\n", humanize.Bytes(uint64(overSizedSize)))

	return partitions, nil
}

// CalculatePartitionsLeftPack calculates the partitions efficiently by searching for files that fit the remaining space in each partition
func CalculatePartitionsLeftPack(store db.DB, targetSize int64) (partitions [][]types.FileMetadata, err error) {
	// TODO: implement this function
	return nil, nil
}
